@@ -11,19 +11,30 @@
 
 module Language.Haskell.GhclibParserEx.Fixity(
     applyFixities
+  , fixitiesFromModule
   , preludeFixities, baseFixities
   , infixr_, infixl_, infix_, fixity
   ) where
 
-import BasicTypes
-#if defined (GHCLIB_API_811) || defined (GHCLIB_API_810)
+#if defined (GHCLIB_API_811)
 import GHC.Hs
-#else
-import HsSyn
-#endif
+import GHC.Types.Basic
+import GHC.Types.Name.Reader
+import GHC.Types.Name
+import GHC.Types.SrcLoc
+#elif defined (GHCLIB_API_810)
+import GHC.Hs
+import BasicTypes
 import RdrName
 import OccName
 import SrcLoc
+#else
+import HsSyn
+import BasicTypes
+import RdrName
+import OccName
+import SrcLoc
+#endif
 import Data.Maybe
 import Data.Data hiding (Fixity)
 import Data.Generics.Uniplate.Data
@@ -48,7 +59,10 @@ expFix _ e = e
 -- LPat and Pat have gone through a lot of churn. See
 -- https://gitlab.haskell.org/ghc/ghc/merge_requests/1925 for details.
 patFix :: [(String, Fixity)] -> LPat GhcPs -> LPat GhcPs
-#if defined (GHCLIB_API_811) || defined (GHCLIB_API_810)
+#if defined (GHCLIB_API_811)
+patFix fixities (L loc (ConPat _ op (InfixCon pat1 pat2))) =
+  L loc (mkConOpPat (getFixities fixities) op (findFixity' (getFixities fixities) op) pat1 pat2)
+#elif defined (GHCLIB_API_810)
 patFix fixities (L loc (ConPatIn op (InfixCon pat1 pat2))) =
   L loc (mkConOpPat (getFixities fixities) op (findFixity' (getFixities fixities) op) pat1 pat2)
 #else
@@ -62,22 +76,38 @@ mkConOpPat ::
   -> Located RdrName -> Fixity
   -> LPat GhcPs -> LPat GhcPs
   -> Pat GhcPs
-#if defined (GHCLIB_API_811) || defined (GHCLIB_API_810)
+#if defined (GHCLIB_API_811)
+mkConOpPat fs op2 fix2 p1@(L loc (ConPat _ op1 (InfixCon p11 p12))) p2
+#elif defined (GHCLIB_API_810)
 mkConOpPat fs op2 fix2 p1@(L loc (ConPatIn op1 (InfixCon p11 p12))) p2
 #else
 mkConOpPat fs op2 fix2 p1@(dL->L loc (ConPatIn op1 (InfixCon p11 p12))) p2
 #endif
+#if defined (GHCLIB_API_811)
+  | nofix_error = ConPat noExtField op2 (InfixCon p1 p2)
+#else
   | nofix_error = ConPatIn op2 (InfixCon p1 p2)
-#if defined (GHCLIB_API_811) || defined (GHCLIB_API_810)
+#endif
+#if defined (GHCLIB_API_811)
+  | associate_right = ConPat noExtField op1 (InfixCon p11 (L loc (mkConOpPat fs op2 fix2 p12 p2)))
+#elif defined (GHCLIB_API_810)
   | associate_right = ConPatIn op1 (InfixCon p11 (L loc (mkConOpPat fs op2 fix2 p12 p2)))
 #else
   | associate_right = ConPatIn op1 (InfixCon p11 (cL loc (mkConOpPat fs op2 fix2 p12 p2)))
 #endif
+#if defined (GHCLIB_API_811)
+  | otherwise = ConPat noExtField op2 (InfixCon p1 p2)
+#else
   | otherwise = ConPatIn op2 (InfixCon p1 p2)
+#endif
   where
     fix1 = findFixity' fs op1
     (nofix_error, associate_right) = compareFixity fix1 fix2
+#if defined (GHCLIB_API_811)
+mkConOpPat _ op _ p1 p2 = ConPat noExtField op (InfixCon p1 p2)
+#else
 mkConOpPat _ op _ p1 p2 = ConPatIn op (InfixCon p1 p2)
+#endif
 
 mkOpApp ::
   [(String, Fixity)]
@@ -177,3 +207,19 @@ infix_  = fixity InfixN
 
 fixity :: FixityDirection -> Int -> [String] -> [(String, Fixity)]
 fixity a p = map (,Fixity (SourceText "") p a)
+
+#if defined (GHCLIB_API_811)
+fixitiesFromModule :: Located HsModule -> [(String, Fixity)]
+#else
+fixitiesFromModule :: Located (HsModule GhcPs) -> [(String, Fixity)]
+#endif
+#if defined(GHCLIB_API_811)
+fixitiesFromModule (L _ (HsModule _ _ _ _ decls _ _)) = concatMap f decls
+#else
+fixitiesFromModule (L _ (HsModule _ _ _ decls _ _)) = concatMap f decls
+#endif
+  where
+    f :: LHsDecl GhcPs -> [(String, Fixity)]
+    f (L _ (SigD _ (FixSig _ (FixitySig _ ops (Fixity _ p dir))))) =
+          fixity dir p (map (occNameString. rdrNameOcc . unLoc) ops)
+    f _ = []
