@@ -3,14 +3,14 @@
 #include "ghclib_api.h"
 module Language.Haskell.GhclibParserEx.GHC.Driver.Flags () where
 
-#if defined(GHCLIB_API_811)
-import GHC.Driver.Session
-#else
+#if !defined(GHCLIB_API_811)
 import DynFlags
 #endif
 
--- Oprhan instance until
--- https://gitlab.haskell.org/ghc/ghc/merge_requests/2905 lands.
+-- This instance landed in
+-- https://gitlab.haskell.org/ghc/ghc/merge_requests/2905.
+#if defined(GHCLIB_API_808) || defined(GHCLIB_API_810)
 instance Bounded Language where
   minBound = Haskell98
   maxBound = Haskell2010
+#endif
