@@ -17,31 +17,49 @@ import Control.Monad
 import Data.List.Extra
 import Data.Maybe
 
-import Language.Haskell.GhclibParserEx.Config
-import Language.Haskell.GhclibParserEx.DynFlags
-import Language.Haskell.GhclibParserEx.Parse
 import Language.Haskell.GhclibParserEx.Dump
 import Language.Haskell.GhclibParserEx.Fixity
+import Language.Haskell.GhclibParserEx.GHC.Settings.Config
+import Language.Haskell.GhclibParserEx.GHC.Parser
+import Language.Haskell.GhclibParserEx.GHC.Hs
 import Language.Haskell.GhclibParserEx.GHC.Hs.ExtendInstances
 import Language.Haskell.GhclibParserEx.GHC.Hs.Expr
 import Language.Haskell.GhclibParserEx.GHC.Hs.Pat
+-- We only test 'isImportQualifiedPost' at this time which requires >=
+-- 8.10; avoid unused import warning.
+#if defined (MIN_VERSION_ghc_lib_parser)
+#  if !MIN_VERSION_ghc_lib_parser(1,  0,  0) || MIN_VERSION_ghc_lib_parser(8, 10, 0)
+import Language.Haskell.GhclibParserEx.GHC.Hs.ImpExp
+#  endif
+#elif __GLASGOW_HASKELL__ >= 810
+import Language.Haskell.GhclibParserEx.GHC.Hs.ImpExp
+#endif
 import Language.Haskell.GhclibParserEx.GHC.Driver.Flags()
+import Language.Haskell.GhclibParserEx.GHC.Driver.Session
+import Language.Haskell.GhclibParserEx.GHC.Types.Name.Reader
 
 #if defined (GHCLIB_API_811) || defined (GHCLIB_API_810)
 import GHC.Hs
-import RdrHsSyn
 #else
 import HsSyn
 #endif
-import SrcLoc
 #if defined (GHCLIB_API_811)
+import GHC.Types.SrcLoc
 import GHC.Driver.Session
+import GHC.Parser.Lexer
+import GHC.Utils.Outputable
+import GHC.Utils.Error
+import GHC.Types.Name.Reader
+import GHC.Types.Name.Occurrence
 #else
+import SrcLoc
 import DynFlags
-#endif
 import Lexer
 import Outputable
 import ErrUtils
+import RdrName
+import OccName
+#endif
 import GHC.LanguageExtensions.Type
 #if defined (GHCLIB_API_808)
 import Bag
@@ -61,6 +79,7 @@ tests = testGroup " All tests"
   , expressionPredicateTests
   , patternPredicateTests
   , dynFlagsTests
+  , nameTests
   ]
 
 makeFile :: FilePath -> String -> IO FilePath
@@ -138,59 +157,56 @@ parseTests = testGroup "Parse tests"
     flags = unsafeGlobalDynFlags
     report flags msgs = concat [ showSDoc flags msg | msg <- pprErrMsgBagWithLoc msgs ]
 
+#if defined(GHCLIB_API_811)
+moduleTest :: String -> DynFlags -> (Located HsModule -> IO ()) -> IO ()
+#else
+moduleTest :: String -> DynFlags -> (Located (HsModule GhcPs) -> IO ()) -> IO ()
+#endif
+moduleTest s flags test =
+      case parseModule s flags of
+        POk _ e -> test e
+        _ -> assertFailure "parse error"
+
 exprTest :: String -> DynFlags -> (LHsExpr GhcPs -> IO ()) -> IO ()
 exprTest s flags test =
       case parseExpression s flags of
-#if defined (GHCLIB_API_811) || defined (GHCLIB_API_810)
-        POk s e ->
-#else
-        POk _ e ->
-#endif
-#if defined (GHCLIB_API_811) || defined (GHCLIB_API_810)
-          case unP (runECP_P e >>= \e -> return e) s :: ParseResult (LHsExpr GhcPs) of
-            POk _  e ->
-#endif
-              test e
-#if defined (GHCLIB_API_811) || defined (GHCLIB_API_810)
-            _ -> assertFailure "parse error"
-#endif
+        POk _ e -> test e
         _ -> assertFailure "parse error"
 
 patTest :: String -> DynFlags -> (LPat GhcPs -> IO ()) -> IO ()
 patTest s flags test =
       case parsePattern s flags of
-#if defined (GHCLIB_API_811) || defined (GHCLIB_API_810)
-        POk _ e ->
-#else
-        POk _ e ->
-#endif
-              test e
+        POk _ e -> test e
         _ -> assertFailure "parse error"
 
 fixityTests :: TestTree
 fixityTests = testGroup "Fixity tests"
-  [ testCase "Expression" $ do
-      let flags = defaultDynFlags fakeSettings fakeLlvmConfig
+  [ testCase "Expression" $
       exprTest "1 + 2 * 3" flags
         (\e ->
             assertBool "parse tree not affected" $
               showSDocUnsafe (showAstData BlankSrcSpan e) /=
               showSDocUnsafe (showAstData BlankSrcSpan (applyFixities [] e))
         )
-  , testCase "Pattern" $ do
-      let flags = defaultDynFlags fakeSettings fakeLlvmConfig
+  , testCase "Pattern" $
       case parseDeclaration "f (1 : 2 :[]) = 1" flags of
         POk _ d ->
           assertBool "parse tree not affected" $
           showSDocUnsafe (showAstData BlankSrcSpan d) /=
           showSDocUnsafe (showAstData BlankSrcSpan (applyFixities [] d))
         PFailed{} -> assertFailure "parse error"
+  , testCase "fixitiesFromModule" $
+      case parseModule "infixl 4 <*!" flags of
+        POk _ m ->
+          assertBool "one fixity expected" $ not (null (fixitiesFromModule m))
+        PFailed{} -> assertFailure "parse error"
   ]
+  where
+    flags = defaultDynFlags fakeSettings fakeLlvmConfig
 
 extendInstancesTests :: TestTree
 extendInstancesTests = testGroup "Extend instances tests"
-  [ testCase "Eq, Ord" $ do
-      let flags = defaultDynFlags fakeSettings fakeLlvmConfig
+  [ testCase "Eq, Ord" $
       exprTest "1 + 2 * 3" flags
         (\e -> do
              e' <- return $ applyFixities [] e
@@ -204,6 +220,8 @@ extendInstancesTests = testGroup "Extend instances tests"
              assertBool ">=" $ e  >= e'
           )
   ]
+  where
+    flags = defaultDynFlags fakeSettings fakeLlvmConfig
 
 expressionPredicateTests :: TestTree
 expressionPredicateTests = testGroup "Expression predicate tests"
@@ -237,7 +255,7 @@ expressionPredicateTests = testGroup "Expression predicate tests"
   , testCase "isDotApp" $ test "f . g" $ assert' . isDotApp
   , testCase "isDotApp" $ test "f $ g" $ assert' . not . isDotApp
   , testCase "isTypeApp" $ test "f @Int" $ assert' . isTypeApp
-#if defined (GHCLIB_API_808)
+#if defined (GHCLIB_API_808) || defined (GHCLIB_API_810)
   , testCase "isTypeApp" $ test "f @ Int" $ assert' . isTypeApp
 #else
   , testCase "isTypeApp" $ test "f @ Int" $ assert' . not . isTypeApp
@@ -265,6 +283,8 @@ expressionPredicateTests = testGroup "Expression predicate tests"
   , testCase "isSpliceDecl" $ test "$(a + b)" $ assert' . isSpliceDecl . unLoc
   , testCase "isQuasiQuote" $ test "[expr|1 + 2|]" $ assert' . isQuasiQuote
   , testCase "isQuasiQuote" $ test "[expr(1 + 2)]" $ assert' . not . isQuasiQuote
+  , testCase "isWholeFrac" $ test "3.2e1" $ assert' . isWholeFrac . unLoc
+  , testCase "isWholeFrac" $ test "3.22e1" $ assert' . not . isWholeFrac . unLoc
   , testCase "strToVar" $ assert' . isVar . strToVar $ "foo"
   , testCase "varToStr" $ test "[]" $ assert' . (== "[]") . varToStr
   , testCase "varToStr" $ test "foo" $ assert' . (== "foo") . varToStr
@@ -281,9 +301,9 @@ patternPredicateTests = testGroup "Pattern predicate tests"
   [ testCase "patToStr" $ test "True" $ assert' . (== "True") . patToStr
   , testCase "patToStr" $ test "False" $ assert' . (== "False") . patToStr
   , testCase "patToStr" $ test "[]" $ assert' . (== "[]") . patToStr
-  , testCase "strToPat" $ assert' . (== "True") . patToStr . noLoc . strToPat $ "True"
-  , testCase "strToPat" $ assert' . (== "False") . patToStr . noLoc . strToPat $ "False"
-  , testCase "strToPat" $ assert' . (== "[]") . patToStr . noLoc . strToPat $ "[]"
+  , testCase "strToPat" $ assert' . (== "True") . patToStr . strToPat $ "True"
+  , testCase "strToPat" $ assert' . (== "False") . patToStr . strToPat $ "False"
+  , testCase "strToPat" $ assert' . (== "[]") . patToStr . strToPat $ "[]"
   , testCase "fromPChar" $ test "'a'" $ assert' . (== Just 'a') . fromPChar
   , testCase "fromPChar" $ test "\"a\"" $ assert' . isNothing . fromPChar
   ]
@@ -301,6 +321,52 @@ dynFlagsTests = testGroup "DynFlags tests"
       Just (_, (es, ds)) <- return $ find (\(e, _) -> e == DeriveTraversable) extensionImplications
       assertBool "no extensions disabled" (null ds)
       assertBool "two extensions enabled" $ DeriveFunctor `elem` es && DeriveFoldable `elem` es
-  , testCase "check instance Bounded Language" $ assertBool "compile time test" (not (null (enumerate @Language)))
-  , testCase "check instace Ord Extension'" $ assertBool "compile time test" (minBound @Extension < maxBound @Extension)
+  , testCase "check instance Bounded Language" $ assertBool "enumerate is null" (not (null (enumerate @Language)))
+  , testCase "check instance Ord Extension" $ assertBool "minBound >= maxBound" (minBound @Extension < maxBound @Extension)
+  , testCase "disable via pragma" $ withTempDir $ \tmpDir -> do
+      foo <- makeFile (tmpDir </> "Foo.hs") $ unlines
+        [ "{-# LANGUAGE NoStarIsType #-}"
+        , "{-# LANGUAGE ExplicitNamespaces #-}"
+        , "import GHC.TypeLits(KnownNat, type (+), type (*))"
+        ]
+      s <- readFile' foo
+      -- If 'StarIsType' ends up enabled after
+      -- 'parsePragmasIntoDynflags' has done its work, we'll get a
+      -- parse error (see
+      -- https://github.com/ndmitchell/hlint/issues/971).
+      parsePragmasIntoDynFlags flags ([StarIsType], []) foo s >>= \case
+        Left msg -> assertFailure msg
+        Right flags -> chkParseResult report flags $ parseFile foo flags s
+#if defined (MIN_VERSION_ghc_lib_parser)
+#  if  !MIN_VERSION_ghc_lib_parser(1,  0,  0) || MIN_VERSION_ghc_lib_parser(8, 10, 0)
+  , testCase "ImportQualifiedPost" $ do
+      case parseImport "import Foo qualified" (flags `xopt_set` ImportQualifiedPost) of
+        POk _ (L _ decl) -> assertBool "expected postpositive" (isImportQualifiedPost . ideclQualified $ decl)
+        PFailed _ -> assertFailure "parse error"
+#  endif
+#elif __GLASGOW_HASKELL__ >= 810
+  , testCase "ImportQualifiedPost" $ do
+      case parseImport "import Foo qualified" (flags `xopt_set` ImportQualifiedPost) of
+        POk _ (L _ decl) -> assertBool "expected postpositive" (isImportQualifiedPost . ideclQualified $ decl)
+        PFailed _ -> assertFailure "parse error"
+#endif
   ]
+  where
+    flags = unsafeGlobalDynFlags
+    report flags msgs = concat [ showSDoc flags msg | msg <- pprErrMsgBagWithLoc msgs ]
+
+nameTests :: TestTree
+nameTests = testGroup "Name tests"
+  [ testCase "modName (1)" $
+      moduleTest "module Foo.Bar.Baz where" flags
+        (\n -> assertBool "Unexpected name string" $ modName n == "Foo.Bar.Baz")
+  , testCase "modName (2)" $
+      moduleTest "f x = x * 2" flags
+        (\n -> assertBool "Unexpected name string" $ modName n == "Main")
+  , testCase "isSymbolRdrName (1)" $ assertBool "Expected 'True'" $ isSymbolRdrName (mkRdrUnqual (mkVarOcc "+"))
+  , testCase "isSymbolRdrName (2)" $ assertBool "Expected 'False'" $ not (isSymbolRdrName (mkRdrUnqual (mkVarOcc "_foo")))
+  , testCase "isSymbolRdrName (3)" $ assertBool "Expected 'False'" $ not (isSymbolRdrName (mkRdrUnqual (mkVarOcc "foo'")))
+  , testCase "isSymbolRdrName (4)" $ assertBool "Expected 'True'" $ isSymbolRdrName (mkRdrUnqual (mkVarOcc ":+:"))
+  ]
+  where
+    flags = defaultDynFlags fakeSettings fakeLlvmConfig
